@@ -189,6 +189,9 @@ class FNN():
     self.weights = [np.random.randn(y, x) * np.sqrt(2 / x) for x, y in zip(self.sizes[:-1], self.sizes[1:])]
     self.biases = [np.random.randn(y, 1) for y in sizes[1:]]
     
+    self.prev_update_w = [np.zeros(w.shape) for w in self.weights]
+    self.prev_update_b = [np.zeros(b.shape) for b in self.biases]
+
     self.epoch_list = np.arange(0, epochs)
 
   def get_params(self):
@@ -329,8 +332,16 @@ class FNN():
 
     loss_grad = loss_function(self.loss_fn, y, activations[-1], True)
     # delta: errors of the output layer
-    delta = loss_grad * activation_function(self.activation_types[-1], zs[-1], True)
-    
+    if (self.loss_fn == "mse"):
+        delta = loss_grad * activation_function(self.activation_types[-1], zs[-1], True)
+    elif (self.loss_fn == "ce"):
+        # if sigmoid or softmax: derivative is out*(1-out) and they cancel each other
+        if (self.activation_types[-1] == "sigmoid" or self.activation_types[-1] == "softmax"):
+            delta = loss_grad
+        else:
+            az = activation_function(self.activation_types[-1], zs[-1], False)
+            delta = loss_grad * (activation_function(self.activation_types[-1], zs[-1], True) / (az * ( 1 - az )))
+
     gradient_w[-1] = np.dot(delta, activations[-2].transpose())
     gradient_b[-1] = delta
     # backpropagate the error
@@ -440,9 +451,13 @@ class FNN():
         gradient_b = [gb + dgb for gb, dgb in zip(gradient_b, delta_gradient_b)]
         gradient_w = [gw + dgw for gw, dgw in zip(gradient_w, delta_gradient_w)]
 
-    update_w = gamma * self.prev_update_w + eta * gradient_w
+    update_w = [o + n for o, n in zip([gamma * puw 
+        for puw in self.prev_update_w], [eta * gw for gw in gradient_w])]
     self.weights = [w - uw for w, uw in zip(self.weights, update_w)]
-    update_b = gamma * self.prev_update_b + eta * gradient_b
+    
+    update_b = [o + n for o, n in zip([gamma * pub for 
+        pub in self.prev_update_b], [eta * gb for gb in gradient_b])]
+    #update_b = gamma * self.prev_update_b + eta * gradient_b
     self.biases = [b - ub for b, ub in zip(self.biases, update_b)]
 
     self.prev_update_w = update_w
@@ -476,8 +491,12 @@ class FNN():
     gradient_b = [np.zeros(b.shape) for b in self.biases]
 
     # w look_ahead partial update
-    update_w = gamma * self.prev_update_w
-    update_b = gamma * self.prev_update_b
+    #update_w = gamma * self.prev_update_w
+    update_w = [o + n for o, n in zip([gamma * puw 
+        for puw in self.prev_update_w], [eta * gw for gw in gradient_w])]
+    #update_b = gamma * self.prev_update_b
+    update_b = [o + n for o, n in zip([gamma * pub 
+        for pub in self.prev_update_b], [eta * gb for gb in gradient_b])]
 
     for x, y in mini_batch:
         delta_gradient_w, delta_gradient_b = self.backprop(x, y, self.weights - update_w, self.biases - update_b)
@@ -485,9 +504,14 @@ class FNN():
         gradient_b = [gb + dgb for gb, dgb in zip(gradient_b, delta_gradient_b)]       
 
     # full update
-    update_w = gamma * self.prev_update_w + eta * gradient_w
+    update_w = [o + n for o, n in zip([gamma * puw 
+        for puw in self.prev_update_w], [eta * gw for gw in gradient_w])]
+    #update_w = gamma * self.prev_update_w + eta * gradient_w
     self.weights = [w - uw for w, uw in zip(self.weights, update_w)]
-    update_b = gamma * self.prev_update_b + eta * gradient_b
+    
+    update_b = [o + n for o, n in zip([gamma * pub
+        for pub in self.prev_update_b], [eta * gb for gb in gradient_b])]
+    #update_b = gamma * self.prev_update_b + eta * gradient_b
     self.biases = [b - ub for b, ub in zip(self.biases, update_b)]
     
     self.prev_update_w = update_w
